use axum::{extract::Path, response::Response};
use serde::Serialize;

use crate::USER;

#[derive(Serialize)]
pub struct UsersPublicKey {
    id: String,
    owner: String,
    #[serde(rename = "publicKeyPem")]
    public_key_pem: String,
}

#[derive(Serialize)]
pub struct UsersResponse {
    #[serde(rename = "@context")]
    context: Vec<String>,
    r#type: String,
    name: String,
    id: String,
    inbox: String,
    outbox: String,
    following: String,
    followers: String,
    summary: String,
    url: String,
    published: String,
    tag: Vec<String>,
    attachment: Vec<String>,
    icon: String,
    #[serde(rename = "preferredUsername")]
    preferred_username: String,
    #[serde(rename = "publicKey")]
    public_key: UsersPublicKey,
}

pub async fn users_get(Path(user): Path<String>) -> Response {
    println!("users: {}", user);
    assert_eq!(user, USER);

    Response::builder()
        .header("content-type", "application/activity+json; charset=utf-8")
        .body(
            serde_json::to_string(&UsersResponse {
                context: vec!["https://www.w3.org/ns/activitystreams".to_string(), "https://w3id.org/security/v1".to_string()],
                r#type: "Person".to_string(),
                name: "Testing Slasher".to_string(),
                id: "https://testing4host.com/users/test71/".to_string(),
                inbox: "https://testing4host.com/inbox/test71/".to_string(),
                outbox: "https://testing4host.com/outbox/test71/".to_string(),
                followers: "https://testing4host.com/followers/test71/".to_string(),
                following: "https://testing4host.com/following/test71/".to_string(),
                summary: "Hi!".to_string(),
                url: "https://testing4host.com/@test71/".to_string(),
                published: "2024-07-13T00:00:00Z".to_string(),
                tag: vec![],
                attachment: vec![],
                icon: "https://testing4host.com/avatar/test71".to_string(),
                preferred_username: "test71".to_string(),
                public_key: UsersPublicKey {
                    id: "https://testing4host.com/users/test71/#main-key".to_string(),
                    owner: "https://testing4host.com/users/test71/".to_string(),
                    public_key_pem: "-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuqubzHLVItM9522mUYSd\nFkpPs0GQalcXVKBFgBCz611fpRzKS5xSGQ8BUVSn/uGkYnOxnh1j41Nk59VUQqCe\nj6d2PUnASbQHc7hcUAPwsgXxZ2LR0z5YTflNIpymd5q3hMqv7NVf05LQnm1t+bbv\nIhiH382J70Ob5FYcae4Shtp5xeBA8IzD9Wo2DwNjH3oP83fybHIEOSofccXB+DI+\np6DZPg2E2/DPpXqVMTmwurlIYX/9jYKYALE9LwX0jY/60lRYkzag9Gr57nlEZH2g\nOOZiy2JmTSvwSWc2LfQyU789w/+b5x9E+lJl1uA4g5HN9F+nEadLRjeMIKyWFdq7\nywIDAQAB\n-----END PUBLIC KEY-----\n".to_string() }
            })
            .unwrap()
            .into(),
        )
        .unwrap()
}
